@@ -1,6 +1,6 @@
-/* Copyright (C) 2000-2008
+/* Copyright (C) 2000-2012
    Stefan Buehler <sbuehler@ltu.se>
-   Patrick Eriksson <Patrick.Eriksson@rss.chalmers.se>
+   Patrick Eriksson <Patrick.Eriksson@chalmers.se>
    Oliver Lemke <olemke@core-dump.info>
 
    This program is free software; you can redistribute it and/or modify it
@@ -100,6 +100,7 @@
 #include <cstdlib>
 #include "matpack.h"
 #include "mystring.h"
+#include "debug.h"
 
 //----------< First of all, include the configuration header >----------
 #include "config_global.h"
@@ -114,12 +115,11 @@
 #endif
 
 //---------------< Global variable declarations >---------------
-// (Definitions of these are in FIXME: where?)
-
+// See global_data.h
 
 //---------------< Global function declarations: >---------------
 // Documentations are with function definitions.
-// FIXME OLE: These should be moved to a separate header file.
+// FIXME: OLE: These should be moved to a separate header file.
 class ArtsOut;
 
 void define_wsv_group_names();  
@@ -132,7 +132,6 @@ void define_lineshape_norm_data();
 
 void arts_exit (int status = EXIT_FAILURE);
 void arts_exit_with_error_message(const String& m, ArtsOut &os);
-void exit_or_rethrow(const String& m, ArtsOut &out);
 
 //
 // Physical constants are now in constants.cc
@@ -140,13 +139,5 @@ void exit_or_rethrow(const String& m, ArtsOut &out);
 
 //---------------< Global macro definitions: >---------------
 
-// Use this macro around function parameter names and variable definitions
-// which are only used in assertions
-#ifndef NDEBUG
-#define DEBUG_ONLY(...) __VA_ARGS__
-#else
-#define DEBUG_ONLY(...)
-#endif
-
 #endif // arts_h
 
