@@ -1,5 +1,5 @@
 
-/* Copyright (C) 2000-2008
+/* Copyright (C) 2000-2012
    Stefan Buehler <sbuehler@ltu.se>
    Patrick Eriksson <patrick.eriksson@chalmers.se>
 
@@ -53,21 +53,40 @@
 // The constants
 ////////////////////////////////////////////////////////////////////////////
 
-/** Global constant, the radius of the Earth [m]
+
+/** Global constant, converts atm to Pa.
+    Multiply your value in atm by this constant to get the value in Pa.
+    \author Stefan Buehler 
+    \date   2000-06-17
+*/
+extern const Numeric ATM2PA         = 1.01325e5;
+
+
+/** Global constant, the Avogadro's number [molec/kg]
     \author Patrick Eriksson 
     \date   08.04.2000
 */
-extern const Numeric EARTH_RADIUS   = 6.378e6;
+extern const Numeric AVOGADROS_NUMB = 6.0220450e26;
 
-/** Global constant, conversion from radians to degrees
-
-    Multiply your value in radians by this constant to get 
-    the value in degrees.
+/** Bohr Magneton
+ *  \author Richard Larsson
+ *  \date   2012-09-26
+ */
+extern const Numeric BOHR_MAGNETON = 9.27400968e-24;
 
+/** Global constant, the Boltzmann constant [J/K]
     \author Patrick Eriksson 
     \date   08.04.2000
 */
-extern const Numeric RAD2DEG        = 57.29577951308232;
+extern const Numeric BOLTZMAN_CONST = 1.380662e-23;
+
+
+/** Global constant, Planck temperature for cosmic background radiation [K]
+    \author Patrick Eriksson 
+    \date   08.04.2000
+*/
+extern const Numeric COSMIC_BG_TEMP = 2.735;
+
 
 /** Global constant, conversion from degrees to radians
 
@@ -77,49 +96,31 @@ extern const Numeric RAD2DEG        = 57.29577951308232;
     \author Patrick Eriksson 
     \date   08.04.2000
 */
-extern const Numeric DEG2RAD        = 0.01745329251994;
+extern const Numeric DEG2RAD        = 0.017453292519943;
 
-/** Global constant, the Planck constant [Js]
+
+/** Global constant, the radius of the Earth [m]
     \author Patrick Eriksson 
     \date   08.04.2000
 */
-extern const Numeric PLANCK_CONST   = 6.626180e-34;
+extern const Numeric EARTH_RADIUS   = 6.3781e6;
+
+
+/** Global constant, the charge of an electron [As]
+    N.B. this is a negative number.
 
-/** Global constant, spped of light in vaccum [m/s]
     \author Patrick Eriksson 
-    \date   08.04.2000
+    \date   2012-04-04
 */
-extern const Numeric SPEED_OF_LIGHT = 2.99792458e8;
+extern const Numeric ELECTRON_CHARGE   = -1.602176565e-19;
 
-/** Global constant, the Boltzmann constant [J/K]
+
+/** Global constant, the mass of an electron [kg]
     \author Patrick Eriksson 
-    \date   08.04.2000
+    \date   2012-04-04
 */
-extern const Numeric BOLTZMAN_CONST = 1.380662e-23;
+extern const Numeric ELECTRON_MASS   = 9.10938291e-31;
 
-/** Global constant, the Avogadro's number [molec/kg]
-    \author Patrick Eriksson 
-    \date   08.04.2000
-*/
-extern const Numeric AVOGADROS_NUMB = 6.0220450e26;
-
-/** Atomic mass unit, 12th of a C^12_6 atom.
-    \author Oliver Lemke
-    \date   27.09.2000
-*/
-extern const Numeric ATOMIC_MASS_UNIT = 1.6606E-27;
-
-/** Global constant, Planck temperature for cosmic background radiation [K]
-    \author Patrick Eriksson 
-    \date   08.04.2000
-*/
-extern const Numeric COSMIC_BG_TEMP = 2.735;
-
-/** Global constant, Planck temperature for solar radiation [K]
-    \author Patrick Eriksson 
-    \date   08.04.2000
-*/
-extern const Numeric SUN_TEMP = 6000.0;
 
 /** Global constant, e (Euler's number)
     \author Thomas Kuhn
@@ -128,17 +129,34 @@ extern const Numeric SUN_TEMP = 6000.0;
 // reference: http://thesapps.com/Doug/exp/
 extern const Numeric EULER_NUMBER   = 2.7182818284590452;
 
+
+/** Global constant, universal gas constant
+    \author Patrick Eriksson
+    \date   2012-03-19
+*/
+extern const Numeric GAS_CONSTANT   = 8.3144621;
+
+
+/** Lande factor gs
+ *  \author Richard Larsson
+ *  \date   2012-09-26
+ */
+// Wikipedia source: http://en.wikipedia.org/wiki/G-factor_%28physics%29#Measured_g-factor_values
+extern const Numeric LANDE_GS      = 2.0023193043622;
+
 /** Global constant, log10(Euler's number)
     \author Thomas Kuhn
     \date   08.11.2001
 */
 extern const Numeric LOG10_EULER_NUMBER = 0.43429448190325176;
 
+
 /** Global constant, ln(10)
     \author Thomas Kuhn
     \date   08.11.2001
 */
-extern const Numeric NAT_LOG_TEN = 2.3025850929940459;
+extern const Numeric NAT_LOG_TEN    = 2.3025850929940459;
+
 
 /** Global constant, ln(2)
     \author Patrick Eriksson 
@@ -146,26 +164,63 @@ extern const Numeric NAT_LOG_TEN = 2.3025850929940459;
 */
 extern const Numeric NAT_LOG_2      = 0.69314718055994;
 
-/** Global constant, sqrt(ln(2))
-    \author Axel von Engeln
-    \date   25.09.2000
-*/
-extern const Numeric SQRT_NAT_LOG_2      = 0.832554611;
 
 /** Global constant, pi
     \author Patrick Eriksson 
     \date   08.04.2000
 */
-extern const Numeric PI             = 3.14159265358979;
+extern const Numeric PI             = 3.141592653589793;
 
-/** Global constant, converts atm to Pa.
 
-    Multiply your value in atm by this constant to get the value in Pa.
-
-    \author Stefan Buehler 
-    \date   2000-06-17
+/** Global constant, the Planck constant [Js]
+    \author Patrick Eriksson 
+    \date   08.04.2000
 */
-extern const Numeric ATM2PA         = 1.01325e5;
+extern const Numeric PLANCK_CONST   = 6.626180e-34;
+
+
+/** Global constant, conversion from radians to degrees
+
+    Multiply your value in radians by this constant to get 
+    the value in degrees.
+
+    \author Patrick Eriksson 
+    \date   08.04.2000
+*/
+extern const Numeric RAD2DEG        = 57.295779513082323;
+
+
+/** Global constant, speed of light in vaccum [m/s]
+    \author Patrick Eriksson 
+    \date   08.04.2000
+*/
+extern const Numeric SPEED_OF_LIGHT = 2.99792458e8;
+
+
+/** Global constant, converts Hz to cm-1.
+
+    Multiply your value in Hz by this constant to get the value in cm-1.
+
+    \author Patrick Eriksson
+    \date   2003-09-07
+*/
+extern const Numeric HZ2CM          =  0.01 / SPEED_OF_LIGHT;
+
+
+/** Global constant, sqrt(ln(2))
+    \author Axel von Engeln
+    \date   25.09.2000
+*/
+extern const Numeric SQRT_NAT_LOG_2 = 0.832554611;
+
+
+/** Global constant, Temperature in Celsius of 0 Kelvin.
+
+    \author Axel von Engeln
+    \date   2000-12-19
+*/
+extern const Numeric TEMP_0_C       =  273.15;
+
 
 /** Global constant, converts torr to Pa.
 
@@ -176,42 +231,26 @@ extern const Numeric ATM2PA         = 1.01325e5;
 */
 extern const Numeric TORR2PA        = 133.3227;
 
-/** Global constant, Temperature in Celsius of 0 Kelvin.
 
-    \author Axel von Engeln
-    \date   2000-12-19
+/** Global constant, the vacuum permittivity [F m-1]
+
+    The constant is also know the permittivity of free space or electric
+    constant.
+
+    \author Patrick Eriksson \date 2012-04-04
 */
-extern const Numeric TEMP_0_C =  273.15;
+extern const Numeric  VACUUM_PERMITTIVITY  = 8.854187817620e-12;
+
+
 
-/** Global constant, Standard pressure in Pa.
 
-    \author Axel von Engeln
-    \date   2000-12-19
-*/
-extern const Numeric PRES_STAND = 101300.25;
 
-/** Global constant,  Loschmidt constant [m^-3].
 
-    \author Axel von Engeln
-    \date   2000-12-19
-*/
-extern const Numeric LOSCHMIDT_CONST = 2.686763E25;
 
-/** Global constant,   Earth gravitational constant [m^3/s^2].
 
-    \author Carlos Jimenez
-    \date   2001-04-20
-*/
-extern const Numeric EARTH_GRAV_CONST = 3.98601E14;
 
-/** Global constant, converts Hz to cm-1.
 
-    Multiply your value in Hz by this constant to get the value in cm-1.
 
-    \author Patrick Eriksson
-    \date   2003-09-07
-*/
-extern const Numeric HZ2CM =  0.01 / SPEED_OF_LIGHT;
 
 /** Global constant, Index of the frequency grid in GriddedField1.
 
@@ -305,8 +344,8 @@ extern const Index GFIELD4_AA_GRID = 3;
 extern const String ABSSPECIES_MAINTAG  = "Absorption species";
 
 extern const String FREQUENCY_MAINTAG   = "Frequency";
-extern const String FREQUENCY_SUBTAG_A  = "Shift + Stretch";
-extern const String FREQUENCY_CALCMODE_A= "Interpolate iyb";
+extern const String FREQUENCY_SUBTAG_0  = "Shift";
+extern const String FREQUENCY_SUBTAG_1  = "Stretch";
 
 extern const String POINTING_MAINTAG    = "Sensor pointing";
 extern const String POINTING_SUBTAG_A   = "Zenith angle off-set";
@@ -314,6 +353,9 @@ extern const String POINTING_CALCMODE_A = "Recalculate iyb";
 extern const String POINTING_CALCMODE_B = "Interpolate iyb";
 
 extern const String POLYFIT_MAINTAG     = "Polynomial baseline fit";
+extern const String SINEFIT_MAINTAG     = "Sinusoidal baseline fit";
 
 extern const String TEMPERATURE_MAINTAG = "Atmospheric temperatures";
 
+extern const String WIND_MAINTAG        = "Wind";
+
