@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2008
+/* Copyright (C) 2001-2012
    Stefan Buehler <sbuehler@ltu.se>
    Mattias Ekstroem <ekstrom@rss.chalmers.se>
 
@@ -57,7 +57,6 @@ public:
   // Constructors:
   Sparse();
   Sparse(Index r, Index c);
-  Sparse(const Sparse& m);
   
   // Insert functions
   void insert_row(Index r, Vector v);
@@ -68,28 +67,22 @@ public:
   // Resize function:
   void resize(Index r, Index c);
 
-  // Destructor:
-  ~Sparse();
-
   // Member functions:
   Index nrows() const;
   Index ncols() const;
   Index nnz()   const;
 
-  const vector<Numeric> * data () const {return mdata;}
-  const vector<Index> * rowind () const {return mrowind;}
-  const vector<Index> * colptr () const {return mcolptr;}
+  const std::vector<Numeric> & data () const {return mdata;}
+  const std::vector<Index> & rowind () const {return mrowind;}
+  const std::vector<Index> & colptr () const {return mcolptr;}
 
   // Index Operators:
   Numeric& rw(Index r, Index c);
   Numeric  ro(Index r, Index c) const;
   Numeric  operator() (Index r, Index c) const;
 
-  // Assignment Operator:
-  Sparse& operator=(const Sparse& m);
-
   // Friends:
-  friend ostream& operator<<(ostream& os, const Sparse& v);
+  friend std::ostream& operator<<(std::ostream& os, const Sparse& v);
   friend void abs (Sparse& A, const Sparse& B );
   friend void mult (VectorView y, const Sparse& M, ConstVectorView x );
   friend void mult (MatrixView A, const Sparse& B, ConstMatrixView C );
@@ -100,11 +93,11 @@ public:
 
 private:
   //! The actual data values.
-  vector<Numeric> *mdata;
+  std::vector<Numeric> mdata;
   //! Row indices.
-  vector<Index> *mrowind;
+  std::vector<Index> mrowind;
   //! Pointers to first data element for each column.
-  vector<Index> *mcolptr;
+  std::vector<Index> mcolptr;
   //! Number of rows in the sparse matrix.
   Index mrr;
   //! Number of rows in the sparse matrix.
