@@ -1,4 +1,4 @@
-/* Copyright (C) 2002-2008 Stefan Buehler <sbuehler@ltu.se>
+/* Copyright (C) 2002-2012 Stefan Buehler <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -23,6 +23,7 @@
 */
 
 #include "matpackVI.h"
+#include "exceptions.h"
 
 // Functions for ConstTensor6View:
 // ------------------------------
@@ -1003,7 +1004,7 @@ ConstTensor6View::ConstTensor6View(Numeric *data,
 /** Output operator. This demonstrates how iterators can be used to
     traverse the tensor. We use the standard output operator for
     Tensor5 to print each page in turn. */
-ostream& operator<<(ostream& os, const ConstTensor6View& v)
+std::ostream& operator<<(std::ostream& os, const ConstTensor6View& v)
 {
   // Page iterators:
   ConstIterator6D ip=v.begin();
@@ -2377,6 +2378,44 @@ VectorView  Tensor6View::operator()
 }
 
 
+/** Conversion to plain C-array.
+    
+This function returns a pointer to the raw data. It fails if the
+Tensor5View is not pointing to the beginning of a Tensor5 or the stride
+is not 1 because the caller expects to get a C array with continuous data.
+*/
+Numeric *Tensor6View::get_c_array()
+{
+    if (mvr.mstart != 0 || mvr.mstride != msr.mextent * mbr.mextent * mpr.mextent * mrr.mextent * mcr.mextent
+        || msr.mstart != 0 || msr.mstride != mbr.mextent * mpr.mextent * mrr.mextent * mcr.mextent
+        || mbr.mstart != 0 || mbr.mstride != mpr.mextent * mrr.mextent * mcr.mextent
+        || mpr.mstart != 0 || mpr.mstride != mrr.mextent * mcr.mextent
+        || mrr.mstart != 0 || mrr.mstride != mcr.mextent
+        || mcr.mstart != 0 || mcr.mstride != 1)
+        throw std::runtime_error("A Tensor6View can only be converted to a plain C-array if it's pointing to a continuous block of data");
+
+    return mdata;
+}
+
+
+/** Conversion to plain C-array.
+
+  This function returns a pointer to the raw data. It fails if the
+  Tensor5View is not pointing to the beginning of a Tensor5 or the stride
+  is not 1 because the caller expects to get a C array with continuous data.
+*/
+const Numeric *Tensor6View::get_c_array() const
+{
+    if (mvr.mstart != 0 || mvr.mstride != msr.mextent * mbr.mextent * mpr.mextent * mrr.mextent * mcr.mextent
+        || msr.mstart != 0 || msr.mstride != mbr.mextent * mpr.mextent * mrr.mextent * mcr.mextent
+        || mbr.mstart != 0 || mbr.mstride != mpr.mextent * mrr.mextent * mcr.mextent
+        || mpr.mstart != 0 || mpr.mstride != mrr.mextent * mcr.mextent
+        || mrr.mstart != 0 || mrr.mstride != mcr.mextent
+        || mcr.mstart != 0 || mcr.mstride != 1)
+        throw std::runtime_error("A Tensor6View can only be converted to a plain C-array if it's pointing to a continuous block of data");
+
+    return mdata;
+}
 
 
 /** Return const iterator to sub-tensor. Has to be redefined here, since it is
@@ -2820,14 +2859,9 @@ Tensor6::Tensor6(const Tensor6& m) :
   \author Stefan Buehler
   \date   2002-12-19
 */
-Tensor6& Tensor6::operator=(const Tensor6& m)
+Tensor6& Tensor6::operator=(Tensor6 x)
 {
-  //  cout << "Tensor6 copy: m = " << m.nrows() << " " << m.ncols() << "\n";
-  //  cout << "             n = " << nrows() << " " << ncols() << "\n";
-
-  resize( m.mvr.mextent, m.msr.mextent, m.mbr.mextent,
-          m.mpr.mextent, m.mrr.mextent, m.mcr.mextent ); 
-  copy( m.begin(), m.end(), begin() );
+  swap(*this, x);
   return *this;
 }
 
@@ -2888,6 +2922,20 @@ void Tensor6::resize(Index v, Index s, Index b,
     }
 }
 
+
+/** Swaps two objects. */
+void swap(Tensor6& t1, Tensor6& t2)
+{
+  std::swap(t1.mvr, t2.mvr);
+  std::swap(t1.msr, t2.msr);
+  std::swap(t1.mbr, t2.mbr);
+  std::swap(t1.mpr, t2.mpr);
+  std::swap(t1.mrr, t2.mrr);
+  std::swap(t1.mcr, t2.mcr);
+  std::swap(t1.mdata, t2.mdata);
+}
+
+
 /** Destructor for Tensor6. This is important, since Tensor6 uses new to
     allocate storage. */
 Tensor6::~Tensor6()
