@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2008 Stefan Buehler <sbuehler@ltu.se>
+/* Copyright (C) 2000-2012 Stefan Buehler <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -35,10 +35,44 @@
 #include <arts_getopt.h>
 #endif
 #include "parameters.h"
+#include "file.h"
 
 /// Holds the command line parameters.
 Parameters parameters;
 
+
+//! Parse path environment variable
+/** 
+ Parse a colon separated list of paths from the given environment variable
+ into an ArrayOfString.
+ 
+ \param[in]  envvar  Name of environment variable.
+ \param[out] paths   ArrayOfString of paths.
+ 
+ \author Oliver Lemke
+ */
+void parse_path_from_environment (String envvar, ArrayOfString& paths)
+{
+  char *envval = getenv (envvar.c_str());
+  if (envval)
+  {
+    String pathstring(envval);
+    
+    // Skip delimiters at beginning.
+    String::size_type lastPos = pathstring.find_first_not_of(":", 0);
+    // Find first "non-delimiter".
+    String::size_type pos     = pathstring.find_first_of(":", lastPos);
+    
+    while (String::npos != pos || String::npos != lastPos)
+    {
+      paths.push_back (pathstring.substr (lastPos, pos - lastPos));
+      lastPos = pathstring.find_first_not_of(":", pos);
+      pos = pathstring.find_first_of(":", lastPos);
+    }
+  }
+}
+
+
 bool get_parameters(int argc, char **argv)
 {
   /*
@@ -96,15 +130,17 @@ bool get_parameters(int argc, char **argv)
 #endif
     { "help",               no_argument,       NULL, 'h' },
     { "includepath",        required_argument, NULL, 'I' },
+    { "datapath",           required_argument, NULL, 'D' },
     { "input",              required_argument, NULL, 'i' },
     { "methods",            required_argument, NULL, 'm' },
     { "numthreads",         required_argument, NULL, 'n' },
+    { "outdir",             required_argument, NULL, 'o' },
     { "plain",              no_argument,       NULL, 'p' },
     { "reporting",          required_argument, NULL, 'r' },
 #ifdef ENABLE_DOCSERVER
     { "docserver",          optional_argument, NULL, 's' },
     { "docdaemon",          optional_argument, NULL, 'S' },
-    { "baseurl",            required_argument, NULL, 'B' },
+    { "baseurl",            required_argument, NULL, 'U' },
 #endif
     { "workspacevariables", required_argument, NULL, 'w' },
     { "version",            no_argument,       NULL, 'v' },
@@ -121,9 +157,11 @@ bool get_parameters(int argc, char **argv)
 #endif
     "       [--help]\n"
     "       [--includepath <path>]\n"
+    "       [--datapath <path>]\n"
     "       [--input <variable>]\n"
     "       [--methods all|<variable>]\n"
     "       [--numthreads <#>\n"
+    "       [--outdir <name>]\n"
     "       [--plain]\n"
     "       [--reporting <xyz>]\n"
 #ifdef ENABLE_DOCSERVER
@@ -156,7 +194,11 @@ bool get_parameters(int argc, char **argv)
     "                    Paths specified on the commandline have precedence\n"
     "                    over the environment variable and will be searched\n"
     "                    first.\n"
-    "-m, --methods       If this is given the argument `all',\n"
+    "-D  --datapath      Additional search path for data files. Directories\n"
+    "                    specified here will be searched after the includepath.\n"
+    "                    Data paths can also be added by setting the\n"
+    "                    environment variable ARTS_DATA_PATH.\n"
+    "-m, --methods       If this is given the argument 'all',\n"
     "                    it simply prints a list of all methods.\n"
     "                    If it is given the name of a variable\n"
     "                    (or variable group), it prints all\n"
@@ -165,6 +207,9 @@ bool get_parameters(int argc, char **argv)
     "-n, --numthreads    If arts was compiled with OpenMP support this option\n"
     "                    can be used to set the maximum number of threads.\n"
     "                    By default OpenMP uses all processors/cores.\n"
+    "-o, --outdir        Set the output directory for the report\n"
+    "                    file and for other output files with relative paths.\n"
+    "                    Default is the current directory.\n"
     "-p  --plain         Generate plain help output suitable for\n"
     "                    script processing.\n"
     "-r, --reporting     Three digit integer. Sets the reporting\n"
@@ -177,12 +222,14 @@ bool get_parameters(int argc, char **argv)
     "                    Default is 010.\n"
 #ifdef ENABLE_DOCSERVER
     "-s, --docserver     Start documentation server. Optionally, specify\n"
-    "                    the port number the server should listen on.\n"
+    "                    the port number the server should listen on,\n"
+    "                    e.g. arts -s9999 or arts --docserver=9999.\n"
     "                    Default is 9000.\n"
     "-S, --docdaemon     Start documentation server in the background.\n"
+    "-U, --baseurl       Base URL for the documentation server.\n"
 #endif
     "-v, --version       Show version information.\n"
-    "-w, --workspacevariables  If this is given the argument `all',\n"
+    "-w, --workspacevariables  If this is given the argument 'all',\n"
     "                    it simply prints a list of all variables.\n"
     "                    If it is given the name of a method, it\n"
     "                    prints all variables needed by this method.";
@@ -238,9 +285,6 @@ bool get_parameters(int argc, char **argv)
         case 'b':
           parameters.basename = optarg;
           break;
-        case 'B':
-          parameters.baseurl = optarg;
-          break;
         case 'd':
           parameters.describe = optarg;
           break;
@@ -256,6 +300,9 @@ bool get_parameters(int argc, char **argv)
         case 'I':
           parameters.includepath.push_back (optarg);
           break;
+        case 'D':
+          parameters.datapath.push_back (optarg);
+          break;
         case 'm':
           parameters.methods = optarg;
           break;
@@ -270,6 +317,9 @@ bool get_parameters(int argc, char **argv)
           }
           break;
         }
+        case 'o':
+          parameters.outdir = optarg;
+          break;
         case 'p':
           parameters.plain = true;
           break;
@@ -324,6 +374,9 @@ bool get_parameters(int argc, char **argv)
             parameters.daemon = true;
             break;
           }
+        case 'U':
+          parameters.baseurl = optarg;
+          break;
         case 'v':
           parameters.version = true;
           break;
@@ -346,38 +399,43 @@ bool get_parameters(int argc, char **argv)
       optind++;
     }
 
-  //  cout << "alle:\n" << parameters.controlfiles << '\n';
-
-  // Look for include paths in the ARTS_PATH environment variable and
+  // Look for include paths in the ARTS_INCLUDE_PATH environment variable and
   // append them to parameters.includepath
 
-  char *artspathenv = getenv ("ARTS_INCLUDE_PATH");
 
-  if (artspathenv)
-    {
-      String artspath (artspathenv);
-
-      // Skip delimiters at beginning.
-      String::size_type lastPos = artspath.find_first_not_of(":", 0);
-      // Find first "non-delimiter".
-      String::size_type pos     = artspath.find_first_of(":", lastPos);
-
-      while (String::npos != pos || String::npos != lastPos)
-        {
-          parameters.includepath.push_back (artspath.substr (lastPos,
-                                                             pos - lastPos));
-          lastPos = artspath.find_first_not_of(":", pos);
-          pos = artspath.find_first_of(":", lastPos);
-        }
-    }
+  parse_path_from_environment(String("ARTS_INCLUDE_PATH"), parameters.includepath);
+  parse_path_from_environment(String("ARTS_DATA_PATH"), parameters.datapath);
   
 #ifdef ARTS_DEFAULT_INCLUDE_DIR
   String arts_default_include_path (ARTS_DEFAULT_INCLUDE_DIR);
   if (arts_default_include_path != "" && !parameters.includepath.nelem())
   {
-    parameters.includepath.push_back (arts_default_include_path);
+    // Skip delimiters at beginning.
+    String::size_type lastPos = arts_default_include_path.find_first_not_of(":", 0);
+    // Find first "non-delimiter".
+    String::size_type pos = arts_default_include_path.find_first_of(":", lastPos);
+    
+    while (String::npos != pos || String::npos != lastPos)
+    {
+      parameters.includepath.push_back (arts_default_include_path.substr (lastPos, pos - lastPos));
+      lastPos = arts_default_include_path.find_first_not_of(":", pos);
+      pos = arts_default_include_path.find_first_of(":", lastPos);
+    }
   }
 #endif
+
+  parameters.includepath.insert(parameters.includepath.begin(), ".");
+  parameters.datapath.insert(parameters.datapath.begin(), ".");
+  
+  if (parameters.outdir.nelem()) 
+    parameters.datapath.insert(parameters.datapath.begin(), parameters.outdir);
+
+  if (parameters.controlfiles.nelem())
+  {
+    String cfdirname;
+    get_dirname(cfdirname, parameters.controlfiles[0]);
+    if (cfdirname.nelem()) parameters.includepath.push_back(cfdirname);
+  }
   
   return false;
 }
