@@ -1,4 +1,4 @@
-/* Copyright (C) 2003-2008 Mattias Ekstrm <ekstrom@rss.chalmers.se>
+/* Copyright (C) 2003-2012 Mattias Ekstrm <ekstrom@rss.chalmers.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -35,6 +35,7 @@
 #include <stdexcept>
 #include "arts.h"
 #include "logic.h"
+#include "make_vector.h"
 #include "matpackI.h"
 #include "matpackII.h"
 #include "messages.h"
@@ -167,14 +168,14 @@ void antenna1d_matrix(
               if( n_ar_f > 1 )
                 {
                   // Interpolation (do this in "green way")
-                  ArrayOfGridPos gp_f( 1 ), gp_za(n_za);
+                  ArrayOfGridPos gp_f( 1 ), gp_za(n_ar_za);
                   gridpos( gp_f, aresponse_f_grid, Vector(1,f_grid[f]) );
                   gridpos( gp_za, aresponse_za_grid, aresponse_za_grid );
-                  Tensor3 itw( 1, n_za, 4 );
+                  Tensor3 itw( 1, n_ar_za, 4 );
                   interpweights( itw, gp_f, gp_za );
-                  Matrix aresponse_matrix(1,n_za);
+                  Matrix aresponse_matrix(1,n_ar_za);
                   interp( aresponse_matrix, itw, 
-                          antenna_response.data(ip,joker,joker,0), gp_f, gp_za );
+                         antenna_response.data(ip,joker,joker,0), gp_f, gp_za );
                   aresponse = aresponse_matrix(0,joker);
                 }
               else if( pol_step )   // Response changes with polarisation
@@ -383,17 +384,20 @@ void antenna2d_simplified(
 
 
 
-//! gaussian_response
+//! gaussian_response_autogrid
 /*!
-   Returns a 1D gaussian response
+   Returns a 1D gaussian response with a suitable grid
 
-   First a grid is generated. The grid is si*[-xwidth_si:dx_si:xwidth_si],
-   where si is the "standard deviation" corresponding to the FWHM.
+   First a grid is generated. The grid is si*[-xwidth_si:dx:xwidth_si],
+   where si is the "standard deviation" corresponding to the FWHM, and
+   dx is biggest possible value < dx_si, to enusre an symmetric grid wth end
+   points exactly at xwidth_si.
    That is, width and spacing of the grid is specified in terms of number of 
    standard deviations. If xwidth_si is set to 2, the response will cover
    about 95% the complete response. For xwidth_si=3, about 99% is covered.
 
-   y is the response matching x.
+   y is the gaussian function on grid x, with max at x0 and width following
+   fwhm.
 
    \param   x           Grid generated.
    \param   y           Calculated response.
@@ -405,21 +409,58 @@ void antenna2d_simplified(
    \author Patrick Eriksson
    \date   2009-09-20
 */
-void gaussian_response(
+void gaussian_response_autogrid(
            Vector&   x,
            Vector&   y,
     const Numeric&   x0,
     const Numeric&   fwhm,
     const Numeric&   xwidth_si,
     const Numeric&   dx_si )
+{
+  assert( dx_si <= xwidth_si );
+
+  const Numeric si = fwhm / ( 2 * sqrt( 2 * NAT_LOG_2 ) );
+
+  // Number of points needed to enure that spacing is max dx_si
+  const Index n = (Index) floor(2*xwidth_si/dx_si) + 1;
+
+  // Grid for response
+  const Numeric dd = si * xwidth_si;
+  nlinspace( x, -dd, dd, n );
+
+  // Calculate response
+  gaussian_response( y, x, x0, fwhm );
+}
+
+
+
+//! gaussian_response
+/*!
+   Returns a 1D gaussian response
+
+   y is the gaussian function on grid x, with max at x0 and width following
+   fwhm.
+
+   \param   y           Calculated response.
+   \param   x           Grid.
+   \param   x0          The x-position of response centre/max.
+   \param   fwhm        The full width at half-maximum of the response
+
+   \author Patrick Eriksson
+   \date   2009-09-20
+*/
+void gaussian_response(
+          Vector&    y,
+    const Vector&    x,
+    const Numeric&   x0,
+    const Numeric&   fwhm )
 {
   const Numeric si = fwhm / ( 2 * sqrt( 2 * NAT_LOG_2 ) );
   const Numeric a = 1 / ( si * sqrt( 2 * PI ) );
+  const Index   n = x.nelem();
 
-  linspace( x, -si*xwidth_si, si*xwidth_si, si*dx_si );
-  const Index n = x.nelem();
   y.resize( n );
-
+  //
   for( Index i=0; i<n; i++ )
     y[i] = a * exp( -0.5 * pow((x[i]-x0)/si,2.0) );
 }
@@ -657,24 +698,23 @@ void sensor_aux_vectors(
 
 //! sensor_integration_vector
 /*!
-   Calculates the (row) vector that multiplied with an unknown
-   (column) vector approximates the integral of the product
-   between the functions represented by the two vectors.
+   Calculates the (row) vector that multiplied with an unknown (column) vector
+   approximates the integral of the product between the functions represented
+   by the two vectors: h*g = integral( f(x)*g(x) dx )
 
-   E.g. h*g = integral( f(x)*g(x) dx )
-
-   See Eriksson et al., Efficient forward modelling by matrix
-   representation of sensor responses, Int. J. Remote Sensing, 27,
-   1793-1808, 2006, for details.
-
-   The grids are internally normalised to cover the range [0,1] for
-   increased numerical stability.
+   Basic principle follows Eriksson et al., Efficient forward modelling by
+   matrix representation of sensor responses, Int. J. Remote Sensing, 27,
+   1793-1808, 2006. However, while in Eriksson et al. the product between f*g
+   is assumed to vary linearly between the grid point, the expressions applied
+   here are more advanced and are completly exact as long as f and g are
+   piece-wise linear functions. The product f*g is then a quadratic funtion
+   between the grid points.
 
    \param   h       The multiplication (row) vector.
    \param   f       The values of function f(x).
    \param   x_f_in  The grid points of function f(x). Must be increasing.
    \param   x_g_in  The grid points of function g(x). Can be increasing or 
-                    decreasing. Must cover a wider range than x_ft (in
+                    decreasing. Must cover a wider range than x_f (in
                     both ends).
 
    \author Mattias Ekstrm and Patrick Eriksson
@@ -697,99 +737,258 @@ void sensor_integration_vector(
   assert( is_increasing( x_g_in ) || is_decreasing( x_g_in ) );
   // More asserts below
 
-  // Copy grids, handle reversed x_g and normalise to cover the range
-  // [0 1]. This is necessary to avoid numerical problems for
-  // frequency grids (e.g. experienced for a case with frequencies
-  // around 501 GHz).
-  //
-  Vector x_g         = x_g_in;
-  Vector x_f         = x_f_in;
+  // End points of x_f
+  Numeric xfmin = x_f_in[0];
+  Numeric xfmax = x_f_in[nf-1];
+
+  // Handle possibly reversed x_g.
+  Vector x_g;
   Index  xg_reversed = 0;
   //
-  if( is_decreasing( x_g ) )
+  if( is_decreasing( x_g_in ) )
     {
+      x_g = x_g_in[Range(ng-1,ng,-1)];
       xg_reversed = 1;
-      Vector tmp  = x_g[Range(ng-1,ng,-1)];   // Flip order
-      x_g         = tmp;
+    }
+  else
+    { 
+      x_g = x_g_in; 
     }
   //
-  assert( x_g[0]    <= x_f[0] );
-  assert( x_g[ng-1] >= x_f[nf-1] );
-  //
-  const Numeric xmin = x_g[0];
-  const Numeric xmax = x_g[ng-1];
-  //
-  x_f -= xmin;
-  x_g -= xmin;
-  x_f /= xmax - xmin;
-  x_g /= xmax - xmin;
+  assert( x_g[0]    <= xfmin );
+  assert( x_g[ng-1] >= xfmax );
 
-  //Create a reference grid vector, x_ref that containing the values
-  //of x_f and x_g strictly sorted. Only g points inside the f range
-  //are of concern.
+  // Normalise grids so x_f covers [0,1]
+  const Numeric df = xfmax - xfmin;
+  Vector x_f(nf);
+  //
+  for( Index i=0; i<nf; i++ )
+    { x_f[i] = ( x_f_in[i] - xfmin ) / df; }
+  for( Index i=0; i<ng; i++ )
+    { x_g[i] = ( x_g[i] - xfmin ) / df; }
+  xfmin = 0;
+  xfmax = 1;
+  // To test without normalisation, comment out lines above and use:
+  //const Numeric df  = 1;
+  //const Vector  x_f = x_f_in;
+  
+  // Create a reference grid vector, x_ref that containing the values
+  // of x_f and x_g strictly sorted. Only g points inside the f range
+  // are of concern.
   list<Numeric> l_x;
   for( Index it=0; it<nf; it++ )
-    l_x.push_back(x_f[it]);
+    { l_x.push_back(x_f[it]); }
   for (Index it=0; it<ng; it++) 
     {
-      if( x_g[it]>x_f[0] && x_g[it]<x_f[x_f.nelem()-1] )
-        l_x.push_back(x_g[it]);
+      if( x_g[it] > xfmin  &&  x_g[it] < xfmax )  
+        { l_x.push_back(x_g[it]); }
     }
-
   l_x.sort();
   l_x.unique();
-
+  //
   Vector x_ref(l_x.size());
-  Index e=0;
-  for (list<Numeric>::iterator li=l_x.begin(); li != l_x.end(); li++) {
-    x_ref[e] = *li;
-    e++;
-  }
+  Index  e = 0;
+  for( list<Numeric>::iterator li=l_x.begin(); li != l_x.end(); li++ ) 
+    {
+      x_ref[e] = *li;
+      e++;
+    }
 
-  //Initiate output vector, with equal size as x_g, with zeros.
-  //Start calculations
+  // Initiate output vector, with equal size as x_g, with zeros.
+  // Start calculations
   h = 0.0;
   Index i_f = 0;
   Index i_g = 0;
-  //i = 0;
   Numeric dx,a0,b0,c0,a1,b1,c1,x3,x2,x1;
-  //while( i_g < ng && i_f < x_f.nelem() ) {
-  for( Index i=0; i<x_ref.nelem()-1; i++ ) {
-    //Find for what index in x_g (which is the same as for h) and f
-    //calculation corresponds to
-    while( x_g[i_g+1] <= x_ref[i] ) {
-      i_g++;
-    }
-    while( x_f[i_f+1] <= x_ref[i] ) {
-     i_f++;
+  //
+  for( Index i=0; i<x_ref.nelem()-1; i++ ) 
+    {
+      // Find for what index in x_g (which is the same as for h) and f
+      // calculation corresponds to
+      while( x_g[i_g+1] <= x_ref[i] ) 
+        { i_g++; }
+      while( x_f[i_f+1] <= x_ref[i] ) 
+        { i_f++; }
+
+      // If x_ref[i] is out of x_f's range then that part of the integral is 0,
+      // and no calculations should be done
+      if( x_ref[i] >= xfmin  &&  x_ref[i] < xfmax ) 
+        {
+          // Product of steps in x_f and x_g
+          dx = ( x_f[i_f+1] - x_f[i_f] ) * ( x_g[i_g+1] - x_g[i_g] );
+
+          // Calculate a, b and c coefficients; h[i]=ax^3+bx^2+cx
+          a0 = ( f[i_f] - f[i_f+1] ) / 3.0;
+          b0 = (-f[i_f]   * ( x_g[i_g+1] + x_f[i_f+1] ) + 
+                 f[i_f+1] * (x_g[i_g+1]+x_f[i_f]) ) / 2.0;
+          c0 = x_g[i_g+1] * ( f[i_f] * x_f[i_f+1] - f[i_f+1] * x_f[i_f] );
+
+          a1 = -a0;
+          b1 = ( f[i_f]   * ( x_g[i_g] + x_f[i_f+1] ) - 
+                 f[i_f+1] * ( x_g[i_g] + x_f[i_f]) ) / 2.0;
+          c1 = x_g[i_g] * ( -f[i_f] * x_f[i_f+1] + f[i_f+1] * x_f[i_f] );
+
+          x1 = x_ref[i+1]-x_ref[i];
+          // Simple way, but sensitive to numerical problems:
+          //x2 = pow(x_ref[i+1],2) - pow(x_ref[i],2);
+          //x3 = pow(x_ref[i+1],3) - pow(x_ref[i],3);
+          // The same but a numerically better way:
+          x2 = x1 * ( 2*x_ref[i] + x1 );
+          x3 = x1 * ( 3*x_ref[i]*(x_ref[i]+x1) + x1*x1 );
+
+          // Calculate h[i] and h[i+1] increment
+          // (df-factor to compensate for normalisation)
+          h[i_g]   += df * ( a0*x3 + b0*x2 + c0*x1 ) / dx;
+          h[i_g+1] += df * ( a1*x3 + b1*x2 + c1*x1 ) / dx;
+        }
     }
 
-    //If x_ref[i] is out of x_f's range then that part of the integral
-    //is set to 0, so no calculations will be done
-    if( x_ref[i] >= x_f[0] && x_ref[i] < x_f[x_f.nelem()-1] ) {
-      //Product of steps in x_f and x_g
-      dx = (x_f[i_f+1] - x_f[i_f]) * (x_g[i_g+1] - x_g[i_g]);
-
-      //Calculate a, b and c coefficients; h[i]=ax^3+bx^2+cx
-      a0 = (f[i_f] - f[i_f+1]) / 3;
-      b0 = (-f[i_f]*(x_g[i_g+1]+x_f[i_f+1])+f[i_f+1]*(x_g[i_g+1]+x_f[i_f]))
-           /2;
-      c0 = f[i_f]*x_f[i_f+1]*x_g[i_g+1]-f[i_f+1]*x_f[i_f]*x_g[i_g+1];
-
-      a1 = -a0;
-      b1 = (f[i_f]*(x_g[i_g]+x_f[i_f+1])-f[i_f+1]*(x_g[i_g]+x_f[i_f]))/2;
-      c1 = -f[i_f]*x_f[i_f+1]*x_g[i_g]+f[i_f+1]*x_f[i_f]*x_g[i_g];
-
-      x3 = pow(x_ref[i+1],3) - pow(x_ref[i],3);
-      x2 = pow(x_ref[i+1],2) - pow(x_ref[i],2);
-      x1 = x_ref[i+1]-x_ref[i];
-
-      //Calculate h[i] and h[i+1] increment
-      h[i_g] += (a0*x3+b0*x2+c0*x1) / dx;
-      h[i_g+1] += (a1*x3+b1*x2+c1*x1) / dx;
+  // Flip back if x_g was decreasing
+  if( xg_reversed )
+    {
+      Vector tmp = h[Range(ng-1,ng,-1)];   // Flip order
+      h = tmp;
+    }
 
+  // The expressions are sensitive to numerical issues if two points in x_ref
+  // are very close compared to the values in x_ref. A test trying to warn when
+  // this happens:
+  if( min(f) >= 0  &&  min(h) < -1e-15 )
+    throw runtime_error( "Significant negative response value obtained, "
+                         "despite sensor reponse is strictly positive. This "
+                         "indicates numerical problems. Is there any very "
+                         "small spacing of the sensor response grid?"
+                         "Please, send a report to Patrick if you see this!" );
+}
+
+//! sensor_integration_vector
+/*!
+   Calculates the (row) vector that multiplied with an unknown
+   (column) vector approximates the integral of the product
+   between the functions represented by the two vectors.
+
+   E.g. h*g = integral( f(x)*g(x) dx )
+
+   See Eriksson et al., Efficient forward modelling by matrix
+   representation of sensor responses, Int. J. Remote Sensing, 27,
+   1793-1808, 2006, for details.
+
+   \param   h       The multiplication (row) vector.
+   \param   f       The values of function f(x).
+   \param   x_f_in  The grid points of function f(x). Must be increasing.
+   \param   x_g_in  The grid points of function g(x). Can be increasing or 
+                    decreasing. Must cover a wider range than x_ft (in
+                    both ends).
+
+   \author Mattias Ekstrm and Patrick Eriksson
+   \date   2003-02-13 / 2008-06-12
+*/
+void sensor_integration_vector2(
+        VectorView   h,
+   ConstVectorView   f,
+   ConstVectorView   x_f,
+   ConstVectorView   x_g_in )
+{
+  // Basic sizes 
+  const Index nf = x_f.nelem();
+  const Index ng = x_g_in.nelem();
+
+  // Asserts
+  assert( h.nelem() == ng );
+  assert( f.nelem() == nf );
+  assert( is_increasing( x_f ) );
+  assert( is_increasing( x_g_in ) || is_decreasing( x_g_in ) );
+  // More asserts below
+
+  // End points of x_f
+  const Numeric xfmin = x_f[0];
+  const Numeric xfmax = x_f[nf-1];
+
+  // Handle possibly reversed x_g.
+  Vector x_g;
+  Index  xg_reversed = 0;
+  //
+  if( is_decreasing( x_g_in ) )
+    {
+      x_g = x_g_in[Range(ng-1,ng,-1)];
+      xg_reversed = 1;
+    }
+  else
+    { 
+      x_g = x_g_in; 
+    }
+  //
+  assert( x_g[0]    <= xfmin );
+  assert( x_g[ng-1] >= xfmax );
+
+  // Create a reference grid vector, x_ref that containing the values
+  // of x_f and x_g strictly sorted. Only g points inside the f range
+  // are of concern.
+  list<Numeric> l_x;
+  for( Index it=0; it<nf; it++ )
+    l_x.push_back(x_f[it]);
+  for( Index it=0; it<ng; it++ ) 
+    {
+      if( x_g[it] > xfmin  &&  x_g[it] < xfmax )
+        { l_x.push_back(x_g[it]); }
+    }
+  l_x.sort();
+  l_x.unique();
+  //
+  Vector x_ref(l_x.size());
+  Index  e = 0;
+  for( list<Numeric>::iterator li=l_x.begin(); li != l_x.end(); li++ ) 
+    {
+      x_ref[e] = *li;
+      e++;
+    }
+
+  // Initiate output vector, with equal size as x_g, with zeros.
+  h = 0.0;
+
+  // Start calculations
+  //
+  Index i_f = 0;
+  Index i_g = 0;
+  //
+  for( Index i=0; i<x_ref.nelem()-1; i++ ) 
+    {
+      // Find for what index in x_g (which is the same as for h) and f
+      // calculation corresponds to
+      while( x_g[i_g+1] <= x_ref[i] ) 
+        { i_g++; }
+      while( x_f[i_f+1] <= x_ref[i] ) 
+        { i_f++; }
+
+      // If x_ref[i] is out of x_f's range then that part of the integral
+      // is  0, and no calculations should be done
+      if( x_ref[i] >= xfmin  &&  x_ref[i] < xfmax ) 
+        {
+          // ( x_k+1 - x_k )/2
+          const Numeric dxk = 0.5 * ( x_ref[i+1] - x_ref[i] );
+
+          // ( x_i+1 - x_i )
+          const Numeric dxi = x_g[i_g+1] - x_g[i_g];
+
+          // The ratio of the two quantities above
+          const Numeric r = dxk / dxi;
+        
+          // Values of f at x_ref[i] and x_ref[i+1]
+          const Numeric dx = x_f[i_f+1] - x_f[i_f];
+          const Numeric f0 = ( f[i_f]   * (x_f[i_f+1]-x_ref[i]) +
+                               f[i_f+1] * (x_ref[i]-x_f[i_f]) ) / dx;
+          const Numeric f1 = ( f[i_f]   * (x_f[i_f+1]-x_ref[i+1]) +
+                               f[i_f+1] * (x_ref[i+1]-x_f[i_f]) ) / dx;
+
+          //cout << "f0/f1 = " << f0-x_ref[i] << " / " << f1-x_ref[i+1] << endl;
+
+          // Add increaments to h
+          h[i_g]   += r * ( f0 * ( x_g[i_g+1] - x_ref[i]   ) + 
+                            f1 * ( x_g[i_g+1] - x_ref[i+1] ) );
+          h[i_g+1] += r * ( f0 * ( x_ref[i]   - x_g[i_g]   ) + 
+                            f1 * ( x_ref[i+1] - x_g[i_g]   ) );
+        }
     }
-  }
 
   // Flip back if x_g was decreasing
   if( xg_reversed )
@@ -965,6 +1164,56 @@ void spectrometer_matrix(
 }
 
 
+
+//! stokes2pol
+/*!
+   Sets up an array of vectors to convert the Stokes vector to different
+   polarsiations.
+
+   The measured value is the sum of the element product of the conversion
+   vector and the Stokes vector. Schematically:
+
+   y[iout] = sum( s2p[i].*iy(iin,joker)
+
+   The order of the vectors follow the coding described for sensor_pol, but
+   zero-based indexing is used here. That is, the first vector (s2p[0])
+   corresponds to I.
+
+   Trailing zeros are not stored, to indicate the required value of
+   *stokes_dim*.
+
+   Vectors for I, Q, U and V are always normalised to have unit length (one
+   value is 1, the remaining ones zero). The first element of remaining vectors
+   is set to nv (and other values normalised accordingly), to allow that
+   calibartion and other normalisation effects can be incorporated.
+
+   \param   s2p           Array of conversion vectors.
+   \param   nv            Norm value for polarisations beside I, Q, U and V.
+
+   \author Patrick Eriksson
+   \date   2011-11-01
+*/
+void stokes2pol( 
+            ArrayOfVector&  s2p,
+      const Numeric&        nv )
+{
+  s2p.resize(10);
+
+  s2p[0] = MakeVector( 1 );               // I
+  s2p[1] = MakeVector( 0, 1 );            // Q
+  s2p[2] = MakeVector( 0, 0, 1 );         // U
+  s2p[3] = MakeVector( 0, 0, 0, 1 );      // V
+  s2p[4] = MakeVector( nv, nv );          // Iv
+  s2p[5] = MakeVector( nv, -nv );         // Ih
+  s2p[6] = MakeVector( nv, 0, nv );       // I+45
+  s2p[7] = MakeVector( nv, 0, -nv );      // I-45
+  s2p[8] = MakeVector( nv, 0, 0, nv );    // Irhc
+  s2p[9] = MakeVector( nv, 0, 0, -nv );   // Ilhc
+}
+
+
+
+
 // Functions by Stefan, needed for HIRS:
 
 
@@ -1082,7 +1331,7 @@ void find_effective_channel_boundaries(// Output:
                                        const Numeric& delta,
                                        const Verbosity& verbosity)
 {
-  CREATE_OUT2
+  CREATE_OUT2;
   
   // How many channels in total:
   const Index n_chan = f_backend.nelem();
@@ -1129,55 +1378,106 @@ void find_effective_channel_boundaries(// Output:
   out2 << "  Original channel characteristics:\n"
        << "  min         nominal      max (all in Hz):\n";
 
+  // count the number of passbands as defined by segments of filtershapes, backend_channel_response.data = [0,>0,>0,0]
+  // Borders between passbands are identified as [...0,0...]
+
   // Get a list of original channel boundaries:
-  Vector fmin_orig(n_chan);
-  Vector fmax_orig(n_chan);  
-  for (Index i=0; i<n_chan; ++i)
-    {
-      // Some handy shortcuts:
-      const Vector& backend_f_grid   = backend_channel_response[i].get_numeric_grid(0);
-//      const Vector& backend_response = backend_channel_response[i];
-      const Index   nf               = backend_f_grid.nelem();
-
-
-      // We have to find the first and last frequency where the
-      // response is actually different from 0. (No point in making
-      // calculations for frequencies where the response is 0.)
-//       Index j=0;
-//       while (backend_response[j] <= 0) ++j;
-//       Numeric bf_min = backend_f_grid[j];
-
-//       j=nf-1;
-//       while (backend_response[j] <= 0) --j;
-//       Numeric bf_max = backend_f_grid[j];
-      //
-      // No, aparently the sensor part want values also where the
-      // response is zero. So we simply take the grid boundaries here.
-      Numeric bf_min = backend_f_grid[0];
-      Numeric bf_max = backend_f_grid[nf-1];
-
-
-      // We need to add a bit of extra margin at both sides,
-      // otherwise there is a numerical problem in the sensor WSMs.
-      //
-      // PE 081003: The accuracy for me (double on 64 bit machine) appears to
-      // be about 3 Hz. Select 1 MHz to have a clear margin. Hopefully OK
-      // for other machines.
-      //
-      // SAB 2010-04-14: The approach with a constant delta does not seem to work 
-      // well in practice. What I do now is that I add a delta corresponding to a 
-      // fraction of the grid spacing. But that is done outside of this function. 
-      // So we set delta = 0 here.
-      //
-      // SAB 2010-05-03: Now we pass delta as a parameter (with a default value of 0).
-
-      fmin_orig[i] = f_backend[i] + bf_min - delta;
-      fmax_orig[i] = f_backend[i] + bf_max + delta;
-
-      out2 << "  " << fmin_orig[i] 
-           << "  " << f_backend[i] 
-           << "  " << fmax_orig[i] << "\n";
+  Index numPB = 0;
+  for(Index idx=0;idx<n_chan;++idx)
+  {
+    const Vector& backend_filter = backend_channel_response[idx].data;
+    if(backend_filter.nelem()>2)
+    { // only run this code when there is more then two elements in the backend
+      for(Index idy =1;idy < backend_filter.nelem();++idy)
+      {
+        if((backend_filter[idy] >0)&& (backend_filter[idy-1]==0))
+        {
+          numPB ++; // Two consecutive zeros gives the border between two passbands
+        }
+      }
     }
+    else 
+    {
+    	numPB ++; 
+    }
+  }
+
+  Vector fmin_pb(numPB);
+  Vector fmax_pb(numPB);
+  Index pbIdx = 0;
+
+  for(Index idx=0;idx<n_chan;++idx) 
+  {
+    // Some handy shortcuts:
+    //
+    // We have to find the first and last frequency where the
+    // response is actually different from 0. (No point in making
+    // calculations for frequencies where the response is 0.)
+    //       Index j=0;
+    //       while (backend_response[j] <= 0) ++j;
+    //       Numeric bf_min = backend_f_grid[j];
+
+    //       j=nf-1;
+    //       while (backend_response[j] <= 0) --j;
+    //       Numeric bf_max = backend_f_grid[j];
+    //
+    // No, aparently the sensor part want values also where the
+    // response is zero. So we simply take the grid boundaries here.
+    //
+    // We need to add a bit of extra margin at both sides,
+    // otherwise there is a numerical problem in the sensor WSMs.
+    //
+    // PE 081003: The accuracy for me (double on 64 bit machine) appears to
+    // be about 3 Hz. Select 1 MHz to have a clear margin. Hopefully OK
+    // for other machines.
+    //
+    // SAB 2010-04-14: The approach with a constant delta does not seem to work 
+    // well in practice. What I do now is that I add a delta corresponding to a 
+    // fraction of the grid spacing. But that is done outside of this function. 
+    // So we set delta = 0 here.
+    //
+    // SAB 2010-05-03: Now we pass delta as a parameter (with a default value of 0).
+    //
+    // Isoz 2013-05-21: Added methods to ignore areas between passbands
+    //
+    const Vector& backend_f_grid   = backend_channel_response[idx].get_numeric_grid(0);
+    const Vector& backend_filter = backend_channel_response[idx].data;
+    if(backend_filter.nelem()>=4)// Is the passband frequency response given explicitly ? e.g. [0,>0,>0,0]
+    {
+      for(Index idy =1;idy < backend_filter.nelem();++idy)
+      {
+        if(idy==1)
+        {
+          fmin_pb[pbIdx] = f_backend[idx]+backend_f_grid[0];
+        }
+        else if((backend_filter[idy] >0) && (backend_filter[idy-1]==0))
+        {
+          fmin_pb[pbIdx]= f_backend[idx] + backend_f_grid[idy-1]-delta;
+        }
+        if((backend_filter[idy] ==0) && (backend_filter[idy-1]>0))
+        {
+          fmax_pb[pbIdx]= f_backend[idx] + backend_f_grid[idy]+delta;
+          out2 << "  " << "fmin_pb "<< fmin_pb[pbIdx] 
+            << "  " << "f_backend" <<f_backend[idx] 
+            << "  " << "fmax_pb "<<fmax_pb[pbIdx] 
+            << "  " << "diff " << fmax_pb[pbIdx]-fmin_pb[pbIdx]
+            << "\n";
+          pbIdx++;
+        }
+      }
+      fmax_pb[pbIdx-1] = f_backend[idx]+last(backend_f_grid);
+    } 
+    else // Or are the passbands given implicitly - such as the default for AMSUA and MHS
+    {
+      fmin_pb[pbIdx]= f_backend[idx] + backend_f_grid[0]-delta ; //delta;
+      fmax_pb[pbIdx]= f_backend[idx] + backend_f_grid[backend_f_grid.nelem()-1]+delta; ///delta;
+      out2 << "  " << "fmin_pb "<< fmin_pb[pbIdx] 
+           << "  " << "f_backend" <<f_backend[pbIdx] 
+           << "  " << "fmax_pb "<<fmax_pb[pbIdx] 
+           << "\n";
+      pbIdx++;
+    }
+  }
 
   // The problem is that channels may be overlapping. In that case, we
   // want to create a frequency grid that covers their entire range,
@@ -1193,17 +1493,17 @@ void find_effective_channel_boundaries(// Output:
   // some HIRS channels.) We sort by the minimum frequency here, not
   // by f_backend. This is necessary for function
   // test_and_merge_two_channels to work correctly. 
-  ArrayOfIndex isorted;  
-  get_sorted_indexes (isorted, fmin_orig);
-
-  fmin.resize(n_chan);
-  fmax.resize(n_chan);
-  for (Index i=0; i<n_chan; ++i)
-    {
-      fmin[i] = fmin_orig[isorted[i]];
-      fmax[i] = fmax_orig[isorted[i]];
-    }
+  ArrayOfIndex isorted;
+  get_sorted_indexes(isorted,fmin_pb);
 
+  fmin.resize(numPB);
+  fmax.resize(numPB);
+  out2 <<" resize numPb "<<numPB<<"\n";
+  for(Index idx = 0;idx<numPB;++idx)
+  {
+    fmin[idx] = fmin_pb[isorted[idx]];
+    fmax[idx] = fmax_pb[isorted[idx]];
+  }
   // We will be testing pairs of channels, and combine them if
   // possible. We have to test always only against the direct
   // neighbour. If that has no overlap, higher channels can not have
@@ -1236,134 +1536,3 @@ void find_effective_channel_boundaries(// Output:
 }
 
 
-
-
-
-
-
-//--- Stuff not yet updated --------------------------------------------------
-
-
-
-
-
-
-
-//! polarisation_matrix
-/*!
-   Sets up the polarisation transfer matrix from stokes vectors describing
-   the sensor polarisation.
-
-   The sensor polarisation matrix is here multiplied 0.5 to get intensities.
-
-   The size of the transfer matrix has to be set up before calling the function
-   as follows:
-     nrows = number of polarisations times frequencies and angles
-     ncols = stokes dimension times frequencies and angles.
-
-   \param   H         The polarisation transfer matrix
-   \param   pol       The polarisation matrix
-   \param   n_f       The number of frequencies
-   \param   n_za      The number of zenith angles/antennas
-   \param   dim       The stokes dimension
-
-   \author Mattias Ekstrm
-   \date   2004-06-02
-
-void polarisation_matrix(
-              Sparse&   H,
-      ConstMatrixView   pol,
-          const Index   n_f,
-          const Index   n_za,
-          const Index   dim )
-{
-  // Assert size of H and pol
-  assert( H.nrows()==pol.nrows()*n_f*n_za );
-  assert( H.ncols()==dim*n_f*n_za );
-  assert( pol.ncols()==dim );
-
-  Index n_pol = pol.nrows();
-  Matrix pol_half = pol;
-  pol_half *= 0.5;
-
-  // Loop over angles
-  for (Index za=0; za<n_za; za++) {
-    //FIXME: Add rotation here?
-
-    // Loop over frequencies
-    for (Index f=0; f<n_f; f++) {
-
-      // Loop over stokes dimensions
-      for (Index d=0; d<dim; d++) {
-
-        // Loop over polarisations
-        for (Index p=0; p<n_pol; p++) {
-
-          if ( pol(p,d)!=0.0 )
-            H.rw(za*n_f*n_pol+f*n_pol+p,za*n_f*dim+f*dim+d)=pol_half(p,d);
-        }
-      }
-    }
-  }
-}
-*/
-
-
-//! rotation_matrix
-/*!
-   Sets up the rotation transfer matrix from the sensor rotation vector.
-
-   The sensor rotation vector should contain the rotation for each
-   direction. It is coupled with the antenna line-of-sight and has to have the
-   same number of elements/rows.
-
-   The size of the transfer matrix has to be set up before calling the function
-   and it is a quadratic matrix with sizes equal the product of stokes_dim and
-   number of antenna line-of-sight (number of rotations).
-
-   \param   H         The polarisation transfer matrix
-   \param   rot       The polarisation matrix
-   \param   n_f       The number of frequencies
-   \param   dim       The stokes dimension
-
-   \author Mattias Ekstrm
-   \date   2004-06-02
-
-void rotation_matrix(
-              Sparse&   H,
-      ConstVectorView   rot,
-          const Index   n_f,
-          const Index   dim )
-{
-  // Assert that the matrix has the right size
-  assert( H.nrows()==H.ncols() );
-  assert( H.nrows()==dim*n_f*rot.nelem() );
-
-  // Setup the L matrix for each rotation and distribute the elements for
-  // all frequencies in the rotation matrix.
-  Matrix L(dim,dim,0.0);
-  if( dim==4 )
-    L(3,3) = 1.0;
-  L(0,0) = 1.0;
-  Index tmp;
-
-  for( Index rit=0; rit<rot.nelem(); rit++ ) {
-    L(1,1) = cos(2*rot[rit]*PI/180.);
-    L(2,2) = L(1,1);
-    L(1,2) = sin(2*rot[rit]*PI/180.);
-    L(2,1) = -L(1,2);
-
-    for( Index fit=0; fit<n_f; fit++ ) {
-      for( Index Lcit=0; Lcit<dim; Lcit++ ) {
-        for( Index Lrit=0; Lrit<dim; Lrit++ ) {
-          tmp = (rit*n_f+fit)*dim;
-          H.rw(tmp+Lrit,tmp+Lcit)=L(Lrit,Lcit);
-        }
-      }
-    }
-  }
-}
-*/
-
-
-
