@@ -1,4 +1,4 @@
-/* Copyright (C) 2002-2008
+/* Copyright (C) 2002-2012
    Stefan Buehler <sbuehler@ltu.se>
    Wolfram-Andre Haas <wolhaas@hermes.fho-emden.de>
 
@@ -23,6 +23,10 @@
 #include "array.h"
 #include "matpackVII.h"
 
+using std::cout;
+using std::setprecision;
+
+
 /* --------------------------------------------------------------
      Helper functions for filling tensors with running numbers,
      starting with start = 1 (default).
