@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2008 Stefan Buehler <sbuehler@ltu.se>
+/* Copyright (C) 2000-2012 Stefan Buehler <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -88,7 +88,7 @@ public:
   operator Matrix() const;
 
   /** Output operator. */
-  friend ostream& operator<<(ostream& os, const TokVal& a);
+  friend std::ostream& operator<<(std::ostream& os, const TokVal& a);
 
 private:
   TokValType    mtype;
